const parse = new DOMParser();
const fs = require("fs");
const path = require("path");

const NotificationController = require("notification-controller");

class NotificationElement extends NotificationController {
    
    constructor(){
        super();
        this.seen = JSON.parse(localStorage.getItem("seen")) || [];
        this.is_set_height = false;
        var style = document.createElement("style");
        style.textContent = fs.readFileSync(path.join(__dirname, "style.css"));
        this.appendChild(style);
        this.template = parse.parseFromString(fs.readFileSync(path.join(__dirname, "template.html")).toString("utf8"), "text/html");
        var t = this.template.querySelector("div.notification-dashboard").cloneNode(true);
        this.appendChild(t);
        t = this.template.querySelector("div.main-page").cloneNode(true);
        this.main_page = this.appendChild(t);
        this.get_run().then(()=>{
            this.isAlarm();
            this.setTemplate();
        });
    }

    async setTemplate(){
        const self = this;
        var main = self.querySelectorAll("div.main-notification");
        if(self.notifications.length == 0) {
            self.querySelector("div.no-notify").classList.add("active");
        } else if(main.length == 0) {
            self.notifications.forEach(function(notify, i){
                self.setMessage(notify, i);
            })
            self.querySelectorAll(".dropdown").forEach(function(element){
                element.addEventListener("click", self.dropdown_click)
            })
            self.querySelectorAll("div.message-main-message button").forEach(function(element){
                element.addEventListener("click", self.button_message_click)
            })
        }        
        this.querySelector("div.close").addEventListener("click", self.notification_return);
        this.querySelector("div.notification-dashboard").addEventListener("click", self.dashboard_click);
    }

    notification_return(e){
        const el = e.target;
        el.parentNode.parentNode.toggleActive();
        el.parentNode.parentNode.removeBackground();
    }
    
    button_message_click(e) {
        window.cep.util.openURLInDefaultBrowser(e.target.getAttribute("link"));
    }

    dropdown_click(e){
        var main = e.target.parentNode.parentNode.parentNode;
        if(main.classList.contains("hide")){
            main.classList.remove("hide");
            main.classList.add("show");
            main.style.height = main.getAttribute("content-height");
        } else {
            main.classList.add("hide");
            main.classList.remove("show");
            main.style.height = "95px";
        }
    }

    setMessage(notify, i){
        var self = this;
        var t = this.template.querySelector("div.main-notification").cloneNode(true);
        var mainNotification = this.querySelector("div.contents").appendChild(t);
        var title = mainNotification.querySelector("div.message-title");
        title.innerHTML = notify.title;
        var datetime = mainNotification.querySelector("div.message-datetime");
        if(self.seen.find(element => element == parseInt(notify.notify_id))) {
            datetime.innerHTML = self.getMonth(notify);
        }

        if(self.seen.find(element => element == parseInt(notify.notify_id)) == undefined){
            mainNotification.querySelector("span.message-new").classList.add("active");
        }

        var mainMessage = mainNotification.querySelector("div.message-main-message > div > p");
        mainMessage.innerHTML = notify.content;
        if(notify.image){
            var img = document.createElement("div");
            img.style.backgroundImage = `url("data:image/png;base64,${notify.image}")`;
            mainNotification.appendChild(img);
        }
        if(notify.has_button_this_notify == "1"){
            var buttonLink = document.createElement("button");
            buttonLink.classList.add("message-button-link");
            buttonLink.innerHTML = notify.button_label;
            buttonLink.setAttribute("link", notify.link);
            mainMessage.appendChild(buttonLink);
        }
        return true;
    }

    isOpenNotification(){
        const self = this;
        this.addBackground();
        if(this.querySelector("div.main-page").classList.contains("active")){
            this.notifications.forEach(function(notify){
                const is_find = self.seen.find(element => element == parseInt(notify.notify_id));
                if(is_find == undefined) {
                    self.seen.push(notify.notify_id);
                }
            })
            localStorage.setItem("seen", JSON.stringify(this.seen));
        }
        if(this.is_set_height == false){
            this.querySelectorAll("div.main-notification").forEach(function(element, i){
                var ch = element.clientHeight;
                if(self.notifications[i].image)
                    ch += 338;
                element.setAttribute("content-height", `${ch}px`);
                element.style.height = i == 0 ? `${ch}px` : "95px";
                element.classList.add(i > 0 ? "hide" : "show");
            })
            this.is_set_height = true;
        }
        return this.seen;
    }

    getMonth(notify){
        const self = this;
        var d = new Date(notify.datetime);
        var months = self.template.querySelectorAll("data[name='months'] li");
        return `${months[d.getMonth()].innerHTML} ${d.getDate()}`
    }

    isAlarm(){
        const self = this;
        this.notifications.forEach(function(notify){
            if(self.seen.find(element => element == parseInt(notify.notify_id)) == undefined){
                document.querySelector("div.new-message").classList.add("active");
                return true;
            }
        })
        return false;
    }

    removeBackground(){
        const main_page = this.querySelector("div.main-page");
        main_page.classList.remove("scrollup");
        setTimeout(() => {
            main_page.classList.remove("active");
        }, 310);
    }
    
    addBackground(){
        const main_page = this.querySelector("div.main-page");
        main_page.classList.add("active");
        setTimeout(() => {
            main_page.classList.add("scrollup");
        }, 310);
    }
    
    dashboard_click(e){
        const el = e.target.parentNode;
        el.querySelector("div.new-message").classList.remove("active");
        el.parentNode.isOpenNotification();
        el.parentNode.toggleActive(el.parentNode.querySelector("div.main-page"));
    }

}

customElements.define("notification-element", NotificationElement);

module.exports = NotificationElement;
