const PixflowControllerAuth = require("pixflow-controller-auth");
const parse = new DOMParser();
const fs = require("fs");
const path = require("path");
const CepManifest = require("pixflow-cep-manifest");
const cep_manifest = new CepManifest();

class PixflowAuthFormEmail extends PixflowControllerAuth {
    constructor(){
        super();
    }

    connectedCallback(){

        const self = this;
        const styles = document.createElement("style");
        styles.textContent = fs.readFileSync(path.join(__dirname, "style.css"));
        this.appendChild(styles);

        this.template = parse.parseFromString(fs.readFileSync(path.join(__dirname, "template.html")).toString("utf8"), "text/html");
        this.appendChild(this.template.querySelector("div"));

        const login_email = this.querySelector("div.login.email");
        const input_email = login_email.querySelector("input.email");
        const btn_send = this.querySelector("button.send");
        const btn_verify = this.querySelector("button.verify");
        const enter_content = this.querySelector("p.enter-content");
        const verify_content = this.querySelector("p.verify-content");
        const email_addr = verify_content.querySelector("span.email-addr");
        const code = this.querySelector("input.code");
        const no_receive = this.querySelector("p.no-receive");
        const change_email = this.querySelector("p.change-email");
        const resend = no_receive.querySelector("span.resend");
        const time_count = no_receive.querySelector("span.time-count");
        const error_valid = this.querySelector("p.error-valid");
        const msg_resend = this.querySelector("p.msg-resend");
        let t;

        const send_timeout = time_cnt => {
            msg_resend.classList.remove("active");
            time_count.classList.add("active");
            time_count.textContent = "";
            t = setInterval(() => {
                time_count.textContent = `${Math.floor(time_cnt / 60)}:${time_cnt % 60}`;
                time_cnt--;
                if(time_cnt < 0){
                    clearInterval(t);
                    time_count.classList.remove("active");
                    code.value = "";
                    resend.classList.add("active");
                    msg_resend.classList.add("active");
                }                        
            }, 1000);
        }

        const verify_click = async e => {
            code.disabled = true;
            btn_verify.disabled = true;
            btn_verify.querySelector("span:nth-child(1)").classList.toggle("active");
            btn_verify.querySelector("span:nth-child(2)").classList.toggle("active");
            const json = await this.api_verify(input_email.value, code.value);
            if(json.status == "Success"){
                this.setLogin(json.data);
            } else if(json.status == "Error"){
                btn_verify.querySelector("span:nth-child(1)").classList.toggle("active");
                btn_verify.querySelector("span:nth-child(2)").classList.toggle("active");
                code.disabled = false;
                btn_verify.disabled = false;
                code.focus();
                code.classList.add("error-valid");
                error_valid.textContent = json.message;
                error_valid.classList.add("active");
            }
        }

        const send_click = async e => {
            input_email.disabled = true;
            btn_send.disabled = true;
            error_valid.classList.remove("active");
            btn_send.querySelector("span:nth-child(1)").classList.toggle("active");
            btn_send.querySelector("span:nth-child(2)").classList.toggle("active");            
            const json = await this.api_send(input_email.value);
            if(json.status == "Success"){
                login_email.classList.remove("active");
                btn_send.classList.remove("active");
                enter_content.classList.remove("active");

                verify_content.classList.add("active");
                code.value = "";
                code.classList.remove("error-valid");
                code.classList.add("active");
                email_addr.textContent = input_email.value;
                btn_verify.classList.add("active");
                no_receive.classList.add("active");
                change_email.classList.add("active");
                send_timeout(json.data.wait);
            } else if(json.status == "Error") {
                input_email.disabled = false;
                btn_send.disabled = false;
                error_valid.classList.add("active");
                error_valid.textContent = json.message;
                btn_send.querySelector("span:nth-child(1)").classList.toggle("active");
                btn_send.querySelector("span:nth-child(2)").classList.toggle("active");
            }
        }

        input_email.addEventListener("input", e => {
            if(e.target.value.length)
                btn_send.disabled = false;
            else
                btn_send.disabled = true;
        })
        
        btn_send.addEventListener("click", send_click)

        input_email.addEventListener("keyup", e => {
            if(e.which == 13)
                send_click(e);
        })

        btn_verify.addEventListener("click", verify_click)

        code.addEventListener("keyup", e => {
            if(e.which == 13)
                verify_click(e);
        })

        code.addEventListener("input", e => {
            const val = e.target.value;
            const len = val.length;
            if(len) {
                if(t._idleTimeout > 0) {
                    btn_verify.disabled = false;
                }
            }
            else
                btn_verify.disabled = true;
        })

        change_email.addEventListener("click", e => {
            verify_content.classList.remove("active");
            code.classList.remove("active");
            email_addr.textContent = input_email.value;
            btn_verify.classList.remove("active");
            no_receive.classList.remove("active");
            change_email.classList.remove("active");
            error_valid.classList.remove("active");
            msg_resend.classList.remove("active");

            login_email.classList.add("active");
            btn_send.classList.add("active");
            enter_content.classList.add("active");
            input_email.disabled = false;
            btn_send.disabled = false;
            clearInterval(t);
        })

        resend.addEventListener("click", async e => {
            const json = await this.api_send(input_email.value);
            if(json){
                error_valid.classList.remove("active");
                code.classList.remove("error-valid");
                resend.classList.remove("active");
                clearInterval(t);
                send_timeout(json.data.wait);
            }
        })
        
    }

}

module.exports = PixflowAuthFormEmail;

customElements.define("auth-form-email", PixflowAuthFormEmail);
