const parse = new DOMParser();
const fs = require("fs");
const path = require("path");

class PixflowBanner extends HTMLElement {
    constructor(){
        super();
    }
    async connectedCallback(){
        const self = this;
        var style = document.createElement("style");
        style.textContent = fs.readFileSync(path.join(__dirname, "style.css"));
        this.appendChild(style);
        this.banner_data = await this.banner_data();
        if(!this.banner_data.title.length)
            return;
        this.classList.add("active");
        this.template = parse.parseFromString(fs.readFileSync(path.join(__dirname, "template.html")).toString("utf8"), "text/html");
        this.appendChild(this.template.querySelector("div"));

        if(this.banner_data.background_image.url.length)
            this.style.backgroundImage = `url(${this.banner_data.background_image.url})`;
        else
            this.style.backgroundColor = this.banner_data.background_color;
        this.querySelector("div > div > p:first-child").innerHTML = this.banner_data.title;
        this.querySelector("div > div > p:last-child").innerHTML = this.banner_data.subtitle;

        const link = this.querySelector("button");
        link.querySelector("span").innerHTML = this.banner_data.cta_title;
        link.addEventListener("click", function(){
            cep.util.openURLInDefaultBrowser(self.banner_data.cta_url);
        })

    }

    banner_data(){
        const init_data = JSON.parse(localStorage.getItem("init-data"));
        const banner_data = init_data.banner_data_v2[`${init_data.userData.subscription_data.free_subscription ? "free" : "premium"}_user`];
        const banner_id = localStorage.getItem("banner_id");
        self.banner_active = localStorage.getItem("banner_active");
        if((banner_data.banner_id != banner_id || self.banner_active != undefined && parseInt(self.banner_active)) || self.banner_active == undefined){
            this.banner_active = 1;
            localStorage.setItem("banner_active", 1);
            localStorage.setItem("banner_id", banner_data.banner_id);
            return banner_data;
        } else {
            return {};
        }
    }

}

customElements.define("pixflow-banner", PixflowBanner);

module.exports = PixflowBanner;
