const fs = require("fs");
const path = require("path");

class PixflowCEPManifest {
    constructor(){
        const parse = new DOMParser();
        const manifest = parse.parseFromString(fs.readFileSync(path.join(__dirname, "..", "..", "csxs/manifest.xml")).toString("utf8"), "text/xml");
        this.extmanifest = manifest.querySelector("ExtensionManifest");
        this.extension_name = this.extmanifest.querySelector("Menu").innerHTML;
        this.item_name = this.get_extension_id().split(".")[2];
        this.extension_version = this.extmanifest.getAttribute("ExtensionBundleVersion");
        this.created_year = this.extmanifest.getAttribute("CreatedYear");
        this.post_id = this.extmanifest.getAttribute("PostId");
        this.main_path = this.extmanifest.querySelector("MainPath").innerHTML;
        this.source = this.extmanifest.getAttribute("Source") || "";
        this.postpace_port_disable = this.extmanifest.hasAttribute("PostpacePortDisable");
        this.pixflow_port_disable = this.extmanifest.hasAttribute("PixflowPortDisable");
        this.asset_bundle_version = this.extmanifest.hasAttribute("AssetBundleVersion") ? this.extmanifest.getAttribute("AssetBundleVersion") : "0.0.0";
        this.bundle_name_market = this.extmanifest.getAttribute("BundleNameMarket") || this.item_name;
        this.certificate_license = this.extmanifest.hasAttribute("CertificateLicense") ? parseInt(this.extmanifest.getAttribute("CertificateLicense")) : 1;
        this.link_market = this.extmanifest.getAttribute("LinkMarket");
    }

    is_bundle_name_market(){
        return this.extmanifest.hasAttribute("BundleNameMarket");
    }

    get_extension_name(){
        return this.extension_name;
    }

    get_item_name(){
        return this.item_name;
    }

    get_extension_version(){
        return this.extension_version;
    }

    get_extension_id(){
        return window.__adobe_cep__.getExtensionId();
    }

    get_created_year(){
        return this.created_year;
    }

    get_post_id(){
        return this.post_id;
    }

    get_main_path(){
        return this.main_path;
    }

    get_source(){
        return this.source;
    }

    get_postpace_port_disable(){
        return this.postpace_port_disable;
    }

    get_pixflow_port_disable(){
        return this.pixflow_port_disable;
    }

    get_asset_bundle_version(){
        return this.asset_bundle_version;
    }

    get_bundle_name_market(){
        return this.bundle_name_market;
    }

    get_link(){
        return this.get_extension_name().replace(/\s/g,"-").toLowerCase();
    }

    get_certificate_license(){
        return this.certificate_license;
    }

    get_link_market() {
        return this.link_market;
    }

    load_init_data(){
        const self = this;
        return new Promise(function(resolve){
            const host_env = JSON.parse(window.__adobe_cep__.getHostEnvironment());
            const query_data = new URLSearchParams({
                user_email_address: localStorage.getItem("email"),
                host: host_env.appName,
            }).toString();
            if(self.port == "postpace"){
                query_data = new URLSearchParams({
                    token: "Qgdq9Le2AjXC3iA6zZ2BdxOhV9",
                    email: localStorage.getItem("email")
                }).toString();
                url = `https://pixflow.net/wp-json/pixflow/check-postpace-user?${query}`;
            }
            const xhr = new XMLHttpRequest();
            xhr.open("post", `https://pixflow.net/wp-json/pixflow/get-mf-plugin-init-data?${query_data}`);
            xhr.setRequestHeader("pxAPIKey", "XxiSMuk@CVkYqJXhq04TevD5qXqsnDVyw3HWw");
            xhr.addEventListener("load", function(){
                const res = this.responseText;
                localStorage.setItem("init-data", res);
                resolve(res);
            })
            xhr.send();        
        })
    }

    async get_init_data(){
        const init_data = localStorage.getItem("init-data");
        if(init_data == null)
            await this.load_init_data();
        return localStorage.getItem("init-data");
    }

}

module.exports = PixflowCEPManifest;