const ReportLogin = require("report-login");
const CepManifest = require("pixflow-cep-manifest");
const cep_manifest = new CepManifest();
const os = require("pixflow-os-platform");
const https = require("https");
const { URLSearchParams } = require("url");

class PixflowControllerAuth extends HTMLElement {
    constructor(){
        super();
        this.cnt = 0;
    }

    redirect_click(e){
        cep.util.openURLInDefaultBrowser(e.target.getAttribute("link"));
    }

    register_click(e){
        const target = e.target;
        let port = target.getAttribute("port");
        const self  = target.parentNode.parentNode.parentNode;
        localStorage.setItem("port", port);
        //self.message_box(`success-${port}`);
        /* self.socket.send(JSON.stringify({
            status: "request",
            login_mode: port
        })) */
    }

    connect(){
        const self = this;
        return new Promise((resolve, reject) => {
            let addr = 'wss://login.pixflow.net';
            if(cep_manifest.get_source() != "Adobe"){
                if(cep_manifest.extmanifest.getAttribute("port") > 0)
                    addr += `:${port}`;
            }
            
            console.log(addr);
            self.socket = new WebSocket(addr);
            self.socket.addEventListener('message', self.socket_message);
            self.socket.addEventListener('error', function(e){
                self.cnt++;
                if(self.cnt == 3){
                    const host_env = JSON.parse(window.__adobe_cep__.getHostEnvironment());
                    const d = new Date();
                    const query = new URLSearchParams({
                        content: [
                            "Unable to Connect To Login Server",
                            [d.getFullYear(), d.getMonth() + 1, d.getDate()].join("-"),
                            [d.getHours(), d.getMinutes(), d.getSeconds()].join(":"),
                            os(),
                            host_env.appName,
                            host_env.appVersion
                        ].join(" - "),
                        username: "PixflowBugReportBot"
                    }).toString();
                    console.log(query);
                    const xhr = new XMLHttpRequest();
                    xhr.open("POST", "https://discord.com/api/webhooks/1326048686953992214/q_Y1v-1MrjW1SCxE-zrddapq_1iVmNf3R0cgdipuzmyhCufvePjimwwSwpdg__fsA0qB", true);
                    xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
                    xhr.addEventListener("load", e => {
                        self.message_box("error");
                    })
                    xhr.send(query);
                } else {
                    self.connect();
                }
            })
            self.socket.addEventListener("open", function(e){
                resolve(true);
            })    
        })
    }

    message_box(mode){
        let message = this.querySelector("div.message");
        let timeout = message.querySelector(`div.${mode} > p`).innerHTML.split(" ").length * 2000;
        message.querySelectorAll("div.active").forEach(function(element){
            element.classList.remove("active");
        })
        let branch = message.querySelector(`div.${mode}`);
        message.classList.add("active");
        branch.classList.add("active");
        if(mode == "error"){
            message.classList.toggle(mode);
        } else {
            setTimeout(function(){
                message.classList.remove("active");
                branch.classList.remove("active");
            }, timeout);    
        }
    }

    async socket_message(event) {
        let json = JSON.parse(event.data);
        switch(json.status){
            case "token":
                cep.util.openURLInDefaultBrowser(`${json.loginUrl}`);
                break;
            case "login":
                this.setLogin(json);
            }
    }

    setLogin(json){
        localStorage.setItem("email", json.user_email);
        localStorage.setItem("md5", json.user_email_hash);
        localStorage.setItem("port", cep_manifest.get_source().toLowerCase());
        if(json.auth_token){
            localStorage.setItem("postpace_user_type", json.postpace_user_type);
            localStorage.setItem("token", json.auth_token);
            localStorage.setItem("first_name", json.first_name);
            localStorage.setItem("last_name", json.last_name);
            localStorage.setItem("avatar", json.avatar);
        }
        const report_login = new ReportLogin();
        report_login.connect().then(() => {
            window.location.href = "./../../index.html?login-page";
        });
    }

    get_user(token){
        return new Promise(function(resolve){
            if(localStorage.getItem("port") == "postpace"){
                var data = JSON.stringify({
                    "query": "query GetUser{getUser{accounts{name thumbnailUrl currentPlan {type}} email firstName thumbnailUrl lastName createdAt  }}"
                  });
                  
                  var xhr = new XMLHttpRequest();
                  xhr.withCredentials = true;
                  
                  xhr.addEventListener("readystatechange", function() {
                    if(this.readyState === 4) {
                      resolve(JSON.parse(this.responseText));
                    }
                  });
                  
                  xhr.open("POST", "https://ppgateway.api.postpace.io/graphql");
                  xhr.setRequestHeader("Content-Type", "application/json");
                  xhr.setRequestHeader("Authorization", `Bearer ${token}`);
                  xhr.send(data);
            }
        });
    }

    get_postpace_address(){
        return new Promise(function(resolve){
            const xhr = new XMLHttpRequest();
            xhr.open("get", "https://pixflow.net/wp-json/pixflow/get-postpace-address");
            xhr.addEventListener("load", function(){
                resolve(this.responseText);
            });
            xhr.send();
        })
    }

    api_send(email){
        return new Promise((resolve, reject) => {
            const xhr = new XMLHttpRequest();
            const query = new URLSearchParams({
                user_email: email
            }).toString();
            xhr.open("POST", `https://pixflow.net/wp-json/pixflow/send-user-plugin-login-token-email?${query}`);
            xhr.addEventListener("load", e => {
                resolve(JSON.parse(e.target.responseText));
            })
            xhr.send();
        })
    }

    api_verify(email, code){
        return new Promise((resolve, reject) => {
            const query = new URLSearchParams({
                user_email: email,
                token: code
            }).toString();
            const xhr = new XMLHttpRequest();
            xhr.open("POST", `https://pixflow.net/wp-json/pixflow/validate-user-plugin-login-token?${query}`);
            xhr.addEventListener("load", e => {
                resolve(JSON.parse(e.target.responseText));
            })
            xhr.send();
        })
    }



}

module.exports = PixflowControllerAuth;
