const parse = new DOMParser();
const fs = require("fs");
const path = require("path");
const https = require("https");
const CepManifest = require("pixflow-cep-manifest");
const platform = require("pixflow-os-platform");
const cep_manifest = new CepManifest();
const AdmZip = require("adm-zip");

class ExtensionUpdate extends HTMLElement {

    constructor(){
        super();
        self = this;
    }

    async connectedCallback(){
        const style = document.createElement("style");
        style.textContent = fs.readFileSync(path.join(__dirname, "style.css"));
        this.appendChild(style);

        this.template = parse.parseFromString(fs.readFileSync(path.join(__dirname, "template.html")).toString("utf8"), "text/html");
        this.appendChild(this.template.querySelector("div"));

        await this.latest_version();
    }

    latest_version(){
        return new Promise(function(resolve){
            const xhr = new XMLHttpRequest();
            xhr.open("get", `https://pixflow.net/wp-json/pixflow/get-plugin-latest-version-tag?repository_name=${cep_manifest.get_extension_id()}`);
            xhr.addEventListener("load", self.package_latest_release);
            xhr.send();    
        })
    }

    package_latest_release(){
        const t = this;
        return new Promise(function(resolve){
            const res = JSON.parse(t.responseText);
            if(res.tag_name > cep_manifest.get_extension_version()){
                self.classList.add("active");
                if(res.body.trim().length){
                    self.git_html_format(res.body);
                    const desc = self.querySelector("div.description");
                    desc.querySelector("div > p > span").innerHTML = res.tag_name;
                    desc.classList.add("active");
                    self.querySelector("div.report").classList.remove("active");
                }
                self.xhr = new XMLHttpRequest();
                const query = new URLSearchParams({
                    repository_name: cep_manifest.get_extension_id(),
                    required_version: res.tag_name,
                    source: cep_manifest.get_source() == "Pixflow" ? "" : cep_manifest.get_source().toLowerCase()
                }).toString();
                self.xhr.open("get", `https://pixflow.net/wp-json/pixflow/get-latest-version-of-plugin-download-url?${query}`);
                self.xhr.addEventListener("load", self.latest_version_from_host);
                self.xhr.send();        
            }
            else
                self.remove();
        })
    }

    git_html_format(str){
        let lines = str.split("\n");
        const list = document.createElement("ul");
        lines.forEach(function(line){
            const item = document.createElement("li");
            if(line.indexOf("- ") >= 0)
                line = line.split("- ")[1];
            item.innerHTML = line;
            list.appendChild(item);
        })
        this.querySelector("div.description > div:last-child").appendChild(list);
        return str;
    }

    latest_version_from_host(e){
        self.update_res = JSON.parse(this.responseText);
        self.update_plugin = self.querySelector("div");
        self.got_it = self.update_plugin.querySelector("button.got-it");
        self.start_update = self.update_plugin.querySelector("button.start-update");
        self.progress = self.update_plugin.querySelector("div.progress");
        self.cancel = self.update_plugin.querySelector("button.cancel");
        self.btn_done = self.update_plugin.querySelector("button.done");
        self.done = self.update_plugin.querySelector("div.done");
        self.report = self.update_plugin.querySelector("div.report");
        self.new_update_icon = self.update_plugin.querySelector("img.new-update-icon");
        self.update_plugin.classList.add("active");
        self.update_plugin.querySelector("div.report span.extension-name").innerHTML = cep_manifest.get_extension_name();
        self.update_plugin.querySelector("div.report span.latest-version").innerHTML = self.update_res.version;
        self.start_update.addEventListener("click", self.start_update_click)
        self.cancel.addEventListener("click", function(){
            if(typeof self.dl_link == "object"){
                self.dl_link.abort();
                self.xhr.abort();
                self.remove();
            }
        })
        self.got_it.addEventListener("click", self.got_it_click);
        self.btn_done.addEventListener("click", self.got_it_click);
    }

    got_it_click(){
        self.remove();
    }

    start_update_click(e){
        e.target.classList.remove("active");
        self.report.classList.remove("active");
        //self.classList.remove("active");
        self.querySelector("div.description").classList.remove("active");
        self.got_it.classList.remove("active");
        self.cancel.classList.add("active");
        const status = self.update_plugin.querySelector("div.status");
        status.classList.add("active");
        self.progress.classList.add("active");
        const bar = self.progress.querySelector("div.bar");
        const percent = self.progress.querySelector("div.percent");
        const downloading = status.querySelector("span.downloading");
        const installing = status.querySelector("span.installing");
        var total_cur = 0;
        const filename = self.update_res.url;
        var filepath = path.join(__dirname, "..", "..", "..", [cep_manifest.get_extension_id(), self.update_res.version].join("-"));
        self.dl_link = https.get(filename, function(response){
            const file = fs.createWriteStream(filepath);
            response.pipe(file);
            const len = parseInt(response.headers["content-length"], 10);
            var cur = 0;
            var loading = 0;
            response.on("data", function(chunk){
                cur += chunk.length;
                loading = parseInt(cur / len * 100);
                const percent_value = `${total_cur + loading}%`;
                percent.innerHTML = percent_value;
                bar.style.width = percent_value;
            })
            file.on("finish", function(){
                total_cur += loading;
                self.cancel.classList.remove("active");
                setTimeout(async () => {
                    self.progress.classList.remove("active");
                    downloading.classList.remove("active");
                    installing.classList.add("active");
                    const extractPath = path.join(__dirname, "..", "..", "..", cep_manifest.get_extension_id());
                    fs.rmdir(extractPath, { recursive: true, force: true }, function(err){
                        const zip = new AdmZip(filepath);
                        zip.extractAllTo(extractPath, true);
                        if(platform() == "Windows")
                            filepath = filepath.replace(/\\/g,"\\\\");
                        window.__adobe_cep__.evalScript(`File("${filepath}").remove()`, function(){});
                        self.report.classList.remove("active");
                        self.new_update_icon.classList.remove("active");
                        status.classList.remove("active");
                        self.progress.classList.remove("active");
                        self.btn_done.classList.add("active");
                        self.done.classList.add("active");    
                    })
                }, 1000);
            })
        })
    }

}

customElements.define("pixflow-extension-update", ExtensionUpdate);

module.exports = ExtensionUpdate;
